#!/usr/bin/env python

# __revision__  = $LastChangedRevision$
# __Copyright__ = Copyright (c) 2007 - 2009, CollabNet, Inc. All rights reserved.

# Note: 
# When loading project data for creating template purpose, following data fields
# are reset/modified:
# planningFolder:  startDate and endDate are set to None
# artifact:        assigned user, if set, is set to nobody.
#                  reportedReleaseId, if set, is set to None
#                  resolvedReleaseId, if set, is set to None

import getopt
import getpass
import os
import signal
import sys

# TODO: proper location for TeamForge dir!
BASEDIR = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))
sys.path.append(BASEDIR)
sys.path.append(os.path.join(BASEDIR, "lib"))

import CTFApp
from ImportModule import Commands
from ImportModule import ApiMethods

__progname__ = 'pitchfork.py'

DEFAULT_URL = 'http://localhost:8080'
DEFAULT_USER = 'admin'
DEFAULT_PASS = 'admin'

# catching interrupts from user gracefully.
def sighandler(signum, stack):
    if signum == signal.SIGINT:
        print >> sys.stderr, "\nReceived Ctrl-C from user, quitting."
    elif signum == signal.SIGHUP:
        print >> sys.stderr, "\nReceived SIGHUP, quitting."
    elif signum == signal.SIGTERM:
        print >> sys.stderr, "\nReceived SIGTERM, quitting."
    project.logoff()
    sys.exit(1)

class Application(object):
    def __init__(self, argv):
        self.session = CTFApp.Session()
        self.base_url = DEFAULT_URL
        self.session.base_url = DEFAULT_URL
        self.debug = False
        self.inputfile = None
        self.interactive = False
        self.outputfile = None
        self.user = DEFAULT_USER
        self.passwd = DEFAULT_PASS
        self.projectId = None
        self.projectName = None
        self.verbose = False
        self.cmd = None
        self.query = None

        self.parse_cmdline(argv)
        self.validate()
        cmd = 'getVersion'
        self.session.Version = ApiMethods[cmd](self.session, cmd, []).export_data()

# undocument command line options:
#            -s | --server base_url    Server base url, eg: http://hostname:port.
#            -u | --user username      Username to connect to TeamForge server.
#            -p | --pwd password       User password.

    def usage(self, msg=None) :
        """print help message"""
        if msg:
            print  "%s" % msg
        print  """ 
    Usage:  %s [-d -v] -c input.tgz | -e output.tgz projectId | -l [querystr] 

    options:
        -h | --help             Print this message.
        -c | --create file.tgz  Create project template from data in file.tgz .
        -e | --export file.tgz  Export project data to file.tgz.
        -i | --interactive      Run script in interactive mode.
        -l | --list [queryStr]  List projectId and title, optionally filter with 
                                queryStr. If no queryStr, list all projects.
        -v | --verbose          Print out verbose output
        -d | --debug            Print out even more verbose output.

    Must choose one of -c, -e, -l options can be used at a time. If none is 
    given, print this message.

    Examples:
      List all projects in TeamForge server:
          pitchfork.py -l

      Exporting project with ID proj1001 to file /tmp/peojct.tgz from localhost 
      using default username and password:
          pitchfork.py -e /tmp/project.tgz proj1001

      Create project template on localhost using default username password, read
      project data from file /tmp/project.tgz :
          pitchfork.py -c /tmp/project.tgz

        """ % __progname__
        sys.exit(1)

    def parse_cmdline(self, argv):
        cmd_args = None
        try:
            opts, cmd_args = getopt.getopt(
               argv, 'hdc:e:ilp:s:u:v', 
                             ['help', 
                              'debug', 
                              'create=', 
                              'export=', 
                              'interactive', 
                              'list', 
                              'pwd=', 
                              'server=', 
                              'user=', 
                              'verbose'])
        except getopt.GetoptError, e:
            self.usage('ERROR: %s' % e.msg)
        for opt, value in opts:
            if (opt in ('-h','--help')):
                self.usage('')
            elif (opt in ('-d', '--debug')):
                self.debug = True
            elif (opt in ('-c', '--create')):
                self.cmd = 'create'
                self.inputfile = str(value)
            elif (opt in ('-e', '--export')):
                self.cmd = 'export'
                self.outputfile = str(value)
            elif (opt in ('-i', '--interactive')):
                self.interactive = True
            elif (opt in ('-l', '--list')):
                self.cmd = 'list'
            elif (opt in ('-s', '--server')):
                self.base_url = str(value)
            elif (opt in ('-u', '--user')):
                self.user = str(value)
            elif (opt in ('-p', '--pwd')):
                self.passwd = str(value)
            elif (opt in ('-v', '--verbose')):
                self.verbose = True

        if self.cmd == 'export' and cmd_args:
            if cmd_args[0].startswith('proj'):
                self.projectId = cmd_args[0]
            else:
                self.projectName = cmd_args[0]
        elif self.cmd == 'list' and cmd_args:
            self.query = cmd_args[0].replace(' ', ' AND ')

        if self.cmd is None:
            self.usage("")

    def validate(self):
        missing = []
        if self.cmd == 'export' and self.projectId is None:
            missing.append('projectId is missing')
        if self.cmd == 'export' and self.outputfile is None:
            missing.append('output filename is missing')
        else:
            self.session.outfile = self.outputfile
        if self.cmd == 'create' and self.inputfile is None:
            missing.append('input filename is missing')
        else:
            self.session.outfile = self.outputfile

        if self.debug:
            self.session.loglevel = 'DEBUG'
        elif self.verbose:
            self.session.loglevel = 'VERBOSE'

        if missing:
            print >> sys.stderr, "ERROR: missing arguments: %s" % '\n'.join(missing)
            print >> sys.stderr, "INFO : Please use -h option for help message"
            sys.exit(1)

        self.session.user = self.user
        self.session.passwd = self.passwd
        Commands['server'](self.session, [self.base_url]).run()
        self.check_server_status()
        self.login()

        if self.session.session_id is None:
            if self.interactive:
                while self.session.session_id is None:
                    print >> sys.stderr, "ERROR: verify username and password: failed"
                    self.get_credential()
                    self.login()
            else:
                print >> sys.stderr, "ERROR: verify username and password: failed"
                print >> sys.stderr, "INFO : run script in interactive mode to enter username and password"
                sys.exit(1)

    def check_server_status(self):
        """Make sure SOAP server is ready before proceeds"""
        import time
        # max waiting period = max_poll X sleep_time = 300 seconds
        retval = None
        max_poll = 60
        sleep_time = 5
        n = 0
        saved_logfile = self.session.logfile
        self.session.logfile = '/dev/null'
        while n < max_poll:
            retval = ApiMethods['getApiVersion'](self.session, 'getApiVersion', []).export_data()
            if  n == 0 and self.session.error > 0:
                if self.session.errmsg and self.session.errmsg.startswith('ERROR'):
                    print >> sys.stderr, "%s" % self.session.errmsg
                    sys.exit(1)
                else:
                    print >> sys.stderr, "Your TeamForge server's web services don't seem to be "\
                                         "ready yet. Web services may take up to 5 minutes to start."
                    print >> sys.stderr, "Optionally, enter Ctrl-C to abort and run this script" \
                                         " again later."
            if retval and retval[0].isdigit():
                if n > 0 :
                    time.sleep(10)
                    print >> sys.stderr, "TeamForge web services are now ready."
                break
            n += 1
            time.sleep(sleep_time)
        if n == max_poll:
            print >> sys.stderr, "TeamForge web services are still not available after 5 minutes." \
                                 " Verify that your site is running, and try this script again."
            sys.exit(1)
        self.session.logfile = saved_logfile
        return

    def login(self):
        ApiMethods['login'](self.session, 'login', [self.session.user, self.session.passwd]).run()
        if self.session.session_id:
            print >> sys.stderr, "INFO : verify username and password: OK"

    def logoff(self):
        if self.session.session_id:
            ApiMethods['logoff'](self.session, 'logoff', [self.session.user, self.session.session_id]).run()
            
    def get_credential(self):
        self.session.user = raw_input("TeamForge site admin username: ")
        self.session.passwd = getpass.getpass("TeamForge site admin password: ")

    def run(self):
        if self.cmd == 'export':
            cmd = 'export'
            argvs = [self.projectId]
            Commands[cmd](self.session, argvs).run()
        elif self.cmd == 'create':
            cmd = 'load'
            argvs = [self.inputfile]
            Commands[cmd](self.session, argvs).run()
        elif self.cmd == 'list':
            if self.query:
                cmd = 'findProjects'
                argvs = [self.query]
            else:
                cmd = 'getProjectList'
                argvs = []
            ApiMethods[cmd](self.session, cmd, argvs).run()
        else:
            self.usage("")

if __name__ == '__main__':

    # setup signal handler to catch Ctrl-C, HUP, TERM signals
    signal.signal(signal.SIGINT, sighandler)
    signal.signal(signal.SIGHUP, sighandler)
    signal.signal(signal.SIGTERM, sighandler)

    project = Application(sys.argv[1:])
    try:
        project.run()
    except Exception as e:
       print >> sys.stderr, "ERROR: Project Template installation failed with below error trace %s" %e.msg
    finally:
        project.logoff()
